/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package FunctionLayer;

import DataLayer.DataMapper;
import FunctionLayer.StanfordParser.SentimentAnalyzerTest;
import com.google.common.base.Stopwatch;
import com.google.common.collect.MapMaker;
import edu.stanford.nlp.ie.AbstractSequenceClassifier;
import edu.stanford.nlp.ie.crf.CRFClassifier;
import edu.stanford.nlp.ling.CoreLabel;
import edu.stanford.nlp.parser.lexparser.LexicalizedParser;
import edu.stanford.nlp.pipeline.Annotation;
import edu.stanford.nlp.pipeline.CoreDocument;
import edu.stanford.nlp.pipeline.StanfordCoreNLP;
import edu.stanford.nlp.tagger.maxent.MaxentTagger;
import edu.stanford.nlp.trees.GrammaticalStructureFactory;
import edu.stanford.nlp.trees.TreebankLanguagePack;
import java.io.IOException;
import java.sql.SQLException;
import java.util.AbstractMap;
import java.util.ArrayList;
import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Properties;
import java.util.concurrent.Callable;
import java.util.concurrent.ConcurrentMap;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ForkJoinPool;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;
import java.util.function.Consumer;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.stream.Collectors;

/**
 *
 * @author install1
 */
public class Datahandler {

    public static final long EXPIRE_TIME_IN_SECONDS = TimeUnit.SECONDS.convert(10, TimeUnit.MINUTES);
    public static final long EXPIRE_TIME_IN_SECONDS1 = TimeUnit.SECONDS.convert(10, TimeUnit.HOURS);
    public static Datahandler instance = new Datahandler();
    private static volatile Double minDistance;
    private static Annotation strAnno;
    private static Annotation strAnnoSentiment;
    private static Annotation strAnnoJMWE;
    private static CoreDocument coreDoc;
    private static ConcurrentMap<String, ConcurrentMap<String, Double>> sentenceRelationMap;
    private static volatile Double preRelationCounters = 0.0;
    private static volatile Double preRelationUserCounters = 0.0;
    private static final ConcurrentMap<Integer, String> stringCache = new MapMaker().concurrencyLevel(2).makeMap();
    private static int positiveRelationCounter = 0;
    private static int negativeRelationCounter = 0;
    private static ConcurrentMap<String, Annotation> pipelineAnnotationCache;
    private static ConcurrentMap<String, Annotation> pipelineSentimentAnnotationCache;
    private static ConcurrentMap<String, Annotation> jmweAnnotationCache;
    private static ConcurrentMap<String, CoreDocument> coreDocumentAnnotationCache;
    private static ConcurrentMap<String, Integer> conversationMatchMap;
    private static ConcurrentMap<String, Integer> conversationUserMatchMap;
    private LinkedHashMap<String, LinkedHashMap<String, Double>> lHMSMX = new LinkedHashMap();
    private final Stopwatch stopwatch;
    private static final ForkJoinPool executor = instantiateExecutor();
    private static String similar = "";
    private static String shiftReduceParserPath = "edu/stanford/nlp/models/srparser/englishSR.ser.gz";
    private static String sentimentModel = "edu/stanford/nlp/models/sentiment/sentiment.ser.gz";
    private static String lexParserEnglishRNN = "edu/stanford/nlp/models/lexparser/englishRNN.ser.gz";
    private static String taggerPath = "edu/stanford/nlp/models/pos-tagger/english-left3words/english-left3words-distsim.tagger";
    private static String nerModel = "edu/stanford/nlp/models/ner/english.all.3class.caseless.distsim.crf.ser.gz";
    private static String nerModel2 = "edu/stanford/nlp/models/ner/english.conll.4class.caseless.distsim.crf.ser.gz";
    private static String nerModel3 = "edu/stanford/nlp/models/ner/english.muc.7class.caseless.distsim.crf.ser.gz";
    private static final String customStopWordList = "start,starts,period,periods,a,an,and,are,as,at,be,but,by,for,if,in,into,is,it,no,not,of,on,or,such,that,the,their,then,there,these,they,this,to,was,will,with";
    private static MaxentTagger tagger;
    private static String[] options = {"-maxLength", "100"};
    private static Properties props = new Properties();
    private static Properties propsSentiment = new Properties();
    private static GrammaticalStructureFactory gsf;
    private static LexicalizedParser lp;
    private static TreebankLanguagePack tlp;
    private static AbstractSequenceClassifier<CoreLabel> classifier;
    // set up Stanford CoreNLP pipeline
    private static final StanfordCoreNLP pipeline = getPipeLineSetUp();
    private static StanfordCoreNLP pipelineSentiment;

    public Datahandler() {
        this.stopwatch = Stopwatch.createUnstarted();
        this.jmweAnnotationCache = new MapMaker().concurrencyLevel(2).makeMap();
        this.pipelineAnnotationCache = new MapMaker().concurrencyLevel(2).makeMap();
        this.pipelineSentimentAnnotationCache = new MapMaker().concurrencyLevel(2).makeMap();
        this.coreDocumentAnnotationCache = new MapMaker().concurrencyLevel(2).makeMap();
        this.conversationMatchMap = new MapMaker().concurrencyLevel(2).makeMap();
        this.sentenceRelationMap = new MapMaker().concurrencyLevel(2).makeMap();
        this.conversationUserMatchMap = new MapMaker().concurrencyLevel(2).makeMap();
    }

    private static StanfordCoreNLP getPipeLineSetUp() {
        props.setProperty("annotators", "tokenize,ssplit,pos,lemma,ner,parse");
        props.setProperty("parse.model", shiftReduceParserPath);
        props.setProperty("parse.maxlen", "90");
        props.setProperty("parse.binaryTrees", "true");
        props.setProperty("threads", "25");
        props.setProperty("pos.maxlen", "90");
        props.setProperty("tokenize.maxlen", "90");
        props.setProperty("ssplit.maxlen", "90");
        props.setProperty("lemma.maxlen", "90");
        props.setProperty("ner.model", nerModel + "," + nerModel2 + "," + nerModel3);
        props.setProperty("ner.combinationMode", "HIGH_RECALL");
        props.setProperty("regexner.ignorecase", "true");
        props.setProperty("ner.fine.regexner.ignorecase", "true");
        props.setProperty("tokenize.options", "untokenizable=firstDelete");
        return new StanfordCoreNLP(props);
    }

    public void shiftReduceParserInitiate() {
        //got 8 cores
        CountDownLatch cdl = new CountDownLatch(2);
        new Thread(() -> {
            try {
                classifier = CRFClassifier.getClassifierNoExceptions(nerModel);
            } catch (ClassCastException ex) {
                Logger.getLogger(Datahandler.class.getName()).log(Level.SEVERE, null, ex);
            }
            cdl.countDown();
        }).start();
        new Thread(() -> {
            propsSentiment.setProperty("parse.model", lexParserEnglishRNN);
            propsSentiment.setProperty("sentiment.model", sentimentModel);
            propsSentiment.setProperty("parse.maxlen", "90");
            propsSentiment.setProperty("threads", "25");
            propsSentiment.setProperty("pos.maxlen", "90");
            propsSentiment.setProperty("tokenize.maxlen", "90");
            propsSentiment.setProperty("ssplit.maxlen", "90");
            propsSentiment.setProperty("annotators", "tokenize,ssplit,pos,parse,sentiment,lemma,stopword"); //coref too expensive memorywise
            propsSentiment.setProperty("customAnnotatorClass.stopword", "FunctionLayer.StopwordAnnotator");
            propsSentiment.setProperty(StopwordAnnotator.STOPWORDS_LIST, customStopWordList);
            propsSentiment.setProperty("tokenize.options", "untokenizable=firstDelete");
            pipelineSentiment = new StanfordCoreNLP(propsSentiment);
            tagger = new MaxentTagger(taggerPath);
            cdl.countDown();
        }).start();
        lp = LexicalizedParser.loadModel(lexParserEnglishRNN, options);
        tlp = lp.getOp().langpack();
        gsf = tlp.grammaticalStructureFactory();
        try {
            cdl.await();
        } catch (InterruptedException ex) {
            System.out.println("cdl await interrupted: " + ex.getLocalizedMessage() + "\n");
        }
        System.out.println("finished shiftReduceParserInitiate\n");
    }

    public static AbstractSequenceClassifier<CoreLabel> getClassifier() {
        return classifier;
    }

    public static void setClassifier(AbstractSequenceClassifier<CoreLabel> classifier) {
        Datahandler.classifier = classifier;
    }

    public void updateStringCache() {
        try {
            checkIfUpdateStrings(true);
        } catch (CustomError ex) {
            Logger.getLogger(Datahandler.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    private static ForkJoinPool instantiateExecutor() {
        //Runtime.getRuntime().availableProcessors() or static value like 25
        return new ForkJoinPool(Runtime.getRuntime().availableProcessors(),
                ForkJoinPool.defaultForkJoinWorkerThreadFactory,
                null, false);
    }

    public static GrammaticalStructureFactory getGsf() {
        return gsf;
    }

    public static StanfordCoreNLP getPipeline() {
        return pipeline;
    }

    public static StanfordCoreNLP getPipelineSentiment() {
        return pipelineSentiment;
    }

    public static MaxentTagger getTagger() {
        return tagger;
    }

    private Map<Integer, String> getCache() throws SQLException, IOException, CustomError {
        return DataMapper.getAllStrings();
    }

    public int getlHMSMXSize() {
        return lHMSMX.size();
    }

    public int getstringCacheSize() {
        return stringCache.size();
    }

    public void initiateMYSQL() throws SQLException, IOException {
        try {
            DataMapper.createTables();
            stringCache.putAll(getCache());
            // lHMSMX = DataMapper.getAllRelationScores();
        } catch (CustomError ex) {
            Logger.getLogger(Datahandler.class
                    .getName()).log(Level.SEVERE, null, ex);
        }
    }

    public void addHLstatsMessages() {
        ConcurrentMap<String, Integer> hlStatsMessages = new MapMaker().concurrencyLevel(2).makeMap();
        ConcurrentMap<Integer, String> strCacheLocal = stringCache;
        Collection<String> strs = DataMapper.getHLstatsMessages().values();
        for (String str : strs) {
            if (hlStatsMessages.get(str) == null) {
                hlStatsMessages.put(str, hlStatsMessages.size());
            }
        }
        int capacity = 35000;
        hlStatsMessages.keySet().forEach(str -> {
            if (!str.startsWith("!") && MessageResponseHandler.getStr().values().size() < capacity) {
                String orElse = strCacheLocal.values().parallelStream().filter(e -> e.equals(str)).findAny().orElse(null);
                if (orElse == null) {
                    MessageResponseHandler.getMessage(str);
                }
            }
        });
    }

    public void instantiateAnnotationMapJMWE() {
        if (!stringCache.isEmpty()) {
            ConcurrentMap<String, Annotation> jmweAnnotation = PipelineJMWESingleton.INSTANCE.getJMWEAnnotation(stringCache.values());
            for (Entry<String, Annotation> entries : jmweAnnotation.entrySet()) {
                jmweAnnotationCache.put(entries.getKey(), entries.getValue());
            }
        }
    }

    public void instantiateAnnotationMap() {
        if (!stringCache.isEmpty()) {
            ConcurrentMap<String, Annotation> Annotationspipeline = new MapMaker().concurrencyLevel(2).makeMap();
            ConcurrentMap<String, Annotation> AnnotationspipelineSentiment = new MapMaker().concurrencyLevel(2).makeMap();
            stringCache.values().parallelStream().forEach(str -> {
                Annotation strAnno = new Annotation(str);
                Annotationspipeline.put(str, strAnno);
                Annotation strAnno2 = new Annotation(str);
                AnnotationspipelineSentiment.put(str, strAnno2);
            });
            ConcurrentMap<String, CoreDocument> coreDocumentpipelineMap = getMultipleCoreDocumentsWaySuggestion(stringCache.values(), pipeline);
            pipeline.annotate(Annotationspipeline.values());
            pipelineSentiment.annotate(AnnotationspipelineSentiment.values());
            Annotationspipeline.entrySet().forEach(pipelineEntry -> {
                pipelineAnnotationCache.put(pipelineEntry.getKey(), pipelineEntry.getValue());
            });
            AnnotationspipelineSentiment.entrySet().forEach(pipelineEntry -> {
                pipelineSentimentAnnotationCache.put(pipelineEntry.getKey(), pipelineEntry.getValue());
            });
            coreDocumentpipelineMap.entrySet().stream().forEach(CD -> {
                coreDocumentAnnotationCache.put(CD.getKey(), CD.getValue());
            });
        }
    }

    private final static ConcurrentMap<Integer, String> cachedReturnEvaluations(ConcurrentMap<String, Double> getMap, ConcurrentMap<Integer, String> strmapreturn, String str) {
        List<Integer> dummy = new ArrayList();
        int relationCap = 20;
        getMap.entrySet().parallelStream().forEach(mapValues -> {
            if (!strmapreturn.values().contains(str) && dummy.isEmpty()) {
                Double scoreRelationNewMsgToRecentMsg = mapValues.getValue();
                System.out.println("strmapreturn size: " + strmapreturn.size() + "\nscoreRelationNewMsgToRecentMsg: "
                        + scoreRelationNewMsgToRecentMsg + "\n");
                if (scoreRelationNewMsgToRecentMsg >= 4100.0) {
                    positiveRelationCounter++;
                    if (positiveRelationCounter > relationCap) {
                        strmapreturn.put(strmapreturn.size() + 1, str);
                    }
                } else if (scoreRelationNewMsgToRecentMsg <= -5000.0) {
                    negativeRelationCounter++;
                    if (negativeRelationCounter > relationCap * 2) {
                        dummy.add(1);
                    }
                }
            }
        });
        return strmapreturn;
    }

    private final static ConcurrentMap<Integer, String> futuresReturnOverallEvaluation(ConcurrentMap<Integer, Future<SimilarityMatrix>> futures, ConcurrentMap<Integer, String> strmapreturn, String str) {
        int relationCap = 20;
        final ConcurrentMap<String, Double> mapUdate = new MapMaker().concurrencyLevel(2).makeMap();
        final List<Integer> dummy = new ArrayList();
        futures.values().parallelStream().forEach(future -> {
            if (!strmapreturn.values().contains(str) && dummy.isEmpty()) {
                final SimilarityMatrix getSMX = retrieveFutureSMX(future);
                if (getSMX != null) {
                    Double scoreRelationNewMsgToRecentMsg = getSMX.getDistance();
                    System.out.println("strmapreturn size: " + strmapreturn.size() + "\nscoreRelationNewMsgToRecentMsg: "
                            + scoreRelationNewMsgToRecentMsg + "\n");
                    mapUdate.put(getSMX.getSecondaryString(), scoreRelationNewMsgToRecentMsg);
                    if (scoreRelationNewMsgToRecentMsg >= 4100.0) {
                        positiveRelationCounter++;
                        if (positiveRelationCounter > relationCap) {
                            strmapreturn.put(strmapreturn.size() + 1, str);
                        }
                    } else if (scoreRelationNewMsgToRecentMsg <= -5000.0) {
                        negativeRelationCounter++;
                        if (negativeRelationCounter > relationCap * 2) {
                            dummy.add(1);
                        }
                    }
                }
            }
        });
        sentenceRelationMap.put(str, mapUdate);
        return strmapreturn;
    }

    private final static ConcurrentMap<Integer, Future<SimilarityMatrix>> StrComparringNoSentenceRelationMap(ConcurrentMap<Integer, String> strCacheLocal,
            String str, ConcurrentMap<String, Annotation> localJMWEMap, ConcurrentMap<String, Annotation> localPipelineAnnotation,
            ConcurrentMap<String, Annotation> localPipelineSentimentAnnotation, ConcurrentMap<String, CoreDocument> localCoreDocumentMap, int strmapSize) {
        final ConcurrentMap<Integer, Future<SimilarityMatrix>> futures = new MapMaker().concurrencyLevel(4).makeMap();
        for (String str1 : strCacheLocal.values()) {
            if (!str.equals(str1)) {
                final SimilarityMatrix SMX = new SimilarityMatrix(str, str1);
                final Callable<SimilarityMatrix> worker;
                if (stringCache.size() < 150) {
                    worker = new SentimentAnalyzerTest(str, str1, SMX,
                            localJMWEMap.get(str), localJMWEMap.get(str1), localPipelineAnnotation.get(str),
                            localPipelineAnnotation.get(str1), localPipelineSentimentAnnotation.get(str),
                            localPipelineSentimentAnnotation.get(str1), localCoreDocumentMap.get(str), localCoreDocumentMap.get(str1));
                } else {
                    worker = new SentimentAnalyzerTest(str, str1, SMX,
                            localJMWEMap.get(str), jmweAnnotationCache.get(str1), localPipelineAnnotation.get(str),
                            pipelineAnnotationCache.get(str1), localPipelineSentimentAnnotation.get(str),
                            pipelineSentimentAnnotationCache.get(str1), localCoreDocumentMap.get(str), coreDocumentAnnotationCache.get(str1));
                }
                futures.put(futures.size() + 1, executor.submit(worker));
                System.out.println("futures.size(): " + futures.size() + "\nstrmap.values().size(): " + strmapSize + "\n");
            }
        }
        return futures;
    }

    private final static ConcurrentMap<Integer, String> stringIteratorComparator(ConcurrentMap<Integer, String> strmap,
            ConcurrentMap<Integer, String> strCacheLocal, ConcurrentMap<String, Annotation> localJMWEMap,
            ConcurrentMap<String, Annotation> localPipelineAnnotation, ConcurrentMap<String, Annotation> localPipelineSentimentAnnotation,
            ConcurrentMap<String, CoreDocument> localCoreDocumentMap) {
        ConcurrentMap<Integer, String> strmapreturn = new MapMaker().concurrencyLevel(4).makeMap();
        for (String str : strmap.values()) {
            final ConcurrentMap<String, Double> getMap = sentenceRelationMap.get(str);
            positiveRelationCounter = 0;
            negativeRelationCounter = 0;
            if (getMap == null) {
                final ConcurrentMap<Integer, Future<SimilarityMatrix>> futures = StrComparringNoSentenceRelationMap(strCacheLocal, str, localJMWEMap, localPipelineAnnotation, localPipelineSentimentAnnotation, localCoreDocumentMap, strmap.size());
                strmapreturn = futuresReturnOverallEvaluation(futures, strmapreturn, str);
            } else {
                strmapreturn = cachedReturnEvaluations(getMap, strmapreturn, str);
            }
        }
        return strmapreturn;
    }

    private static ConcurrentMap<Integer, String> removeNonSensicalStrings(ConcurrentMap<Integer, String> strmap) {
        final ConcurrentMap<Integer, String> strCacheLocal = stringCache.size() < 150 ? strmap : stringCache;
        final ConcurrentMap<String, Annotation> localJMWEMap = getMultipleJMWEAnnotation(strmap.values());
        final ConcurrentMap<String, Annotation> localPipelineAnnotation = getMultiplePipelineAnnotation(strmap.values());
        final ConcurrentMap<String, Annotation> localPipelineSentimentAnnotation = getMultiplePipelineSentimentAnnotation(strmap.values());
        final ConcurrentMap<String, CoreDocument> localCoreDocumentMap = getMultipleCoreDocumentsWaySuggestion(strmap.values(), pipeline);
        System.out.println("finished removeNonSensicalStrings annotations \n");
        return stringIteratorComparator(strmap, strCacheLocal, localJMWEMap, localPipelineAnnotation, localPipelineSentimentAnnotation, localCoreDocumentMap);
    }

    private static SimilarityMatrix retrieveFutureSMX(Future<SimilarityMatrix> future) {
        try {
            return future.get(5, TimeUnit.SECONDS);
        } catch (InterruptedException | ExecutionException | TimeoutException ex) {
            Logger.getLogger(Datahandler.class.getName()).log(Level.SEVERE, null, ex);
        }
        return null;
    }

    public synchronized void checkIfUpdateStrings(boolean hlStatsMsg) throws CustomError {
        if (stopwatch.elapsed(TimeUnit.SECONDS) >= EXPIRE_TIME_IN_SECONDS || !stopwatch.isRunning()) {
            ConcurrentMap<Integer, String> str = MessageResponseHandler.getStr();
            str = cutContent(str, hlStatsMsg);
            str = filterContent(str);
            str = removeSlacks(str);
            System.out.println("finished removeSlacks \n" + str.size() + "\n");
            str = removeNonSensicalStrings(str);
            System.out.println("POST removeNonSensicalStrings size: " + str.size() + "\n");
            str = annotationCacheUpdate(str);
            System.out.println("annotationCacheUpdate str size POST: " + str.size() + "\n");
            ConcurrentMap<Integer, String> strf = str;
            if (!stringCache.isEmpty()) {
                new Thread(() -> {
                    try {
                        DataMapper.InsertMYSQLStrings(strf);
                    } catch (CustomError ex) {
                        Logger.getLogger(Datahandler.class
                                .getName()).log(Level.SEVERE, null, ex);
                    }
                    MessageResponseHandler.setStr(new MapMaker().concurrencyLevel(2).makeMap());
                }).start();
            } else {
                try {
                    DataMapper.InsertMYSQLStrings(strf);
                } catch (CustomError ex) {
                    Logger.getLogger(Datahandler.class
                            .getName()).log(Level.SEVERE, null, ex);
                }
                MessageResponseHandler.setStr(new MapMaker().concurrencyLevel(2).makeMap());
            }
            if (!stopwatch.isRunning()) {
                stopwatch.start();
            } else {
                stopwatch.reset();
            }
        }
    }

    /**
     *
     * @param str is strF aka user message,
     * @param MostRecent String most recently responded with
     * @return
     * @throws CustomError
     */
    public synchronized String getResponseMsg(String str, String MostRecent) throws CustomError {
        str = str.trim();
        if (str.startsWith("<@")) {
            str = str.substring(str.indexOf("> ") + 2);
        }
        ConcurrentMap<Integer, String> strCache = stringCache;
        ConcurrentMap<Integer, Future<SimilarityMatrix>> futureslocal = new MapMaker().concurrencyLevel(2).makeMap();
        ConcurrentMap<Integer, SimilarityMatrix> futureAndCacheCombineMap = new MapMaker().concurrencyLevel(2).makeMap();
        String strF = str;
        getSingularAnnotation(strF);
        if (!conversationMatchMap.keySet().contains(MostRecent) && !MostRecent.isEmpty()) {
            conversationMatchMap.put(MostRecent, conversationMatchMap.size());
        }
        if (!conversationUserMatchMap.keySet().contains(strF)) {
            conversationUserMatchMap.put(strF, conversationUserMatchMap.size());
        }
        ConcurrentMap<String, Double> getPrimary = sentenceRelationMap.get(strF);
        strCache.values().parallelStream().forEach((str1) -> {
            if (!strF.equals(str1)) {
                boolean present = false;
                if (getPrimary != null) {
                    Double getSecondary = getPrimary.get(str1);
                    if (getSecondary != null) {
                        present = true;
                    }
                }
                if (!present) {
                    SimilarityMatrix SMX = new SimilarityMatrix(strF, str1);
                    Callable<SimilarityMatrix> worker = new SentimentAnalyzerTest(strF, str1, SMX,
                            strAnnoJMWE, jmweAnnotationCache.get(str1), strAnno,
                            pipelineAnnotationCache.get(str1), strAnnoSentiment,
                            pipelineSentimentAnnotationCache.get(str1), coreDoc, coreDocumentAnnotationCache.get(str1));
                    futureslocal.put(futureslocal.size() + 1, executor.submit(worker));
                }
            }
        });
        futureslocal.values().forEach((future) -> {
            SimilarityMatrix SMX = new SimilarityMatrix("", "");
            try {
                SMX = future.get(5, TimeUnit.SECONDS);
                futureAndCacheCombineMap.put(futureAndCacheCombineMap.size(), SMX);
            } catch (InterruptedException | ExecutionException | TimeoutException ex) {
                System.out.println("ex getResponsemsg: " + ex.getMessage() + "\n");
            }
        });
        if (getPrimary != null) {
            for (Entry<String, Double> cacheResults : getPrimary.entrySet()) {
                SimilarityMatrix SMX = new SimilarityMatrix(strF, cacheResults.getKey());
                SMX.setDistance(cacheResults.getValue());
                futureAndCacheCombineMap.put(futureAndCacheCombineMap.size(), SMX);
            }
        }
        futureAndCacheCombineMap.values().parallelStream().forEach((SMX) -> {
            if (sentenceRelationMap.get(strF) == null) {
                ConcurrentMap<String, Double> localMap = new MapMaker().concurrencyLevel(2).makeMap();
                localMap.put(SMX.getSecondaryString(), SMX.getDistance());
                sentenceRelationMap.put(SMX.getPrimaryString(), localMap);
            } else {
                ConcurrentMap<String, Double> getPrimaryLocal = sentenceRelationMap.get(strF);
                Double doubleValue = getPrimaryLocal.get(SMX.getSecondaryString());
                if (doubleValue == null) {
                    getPrimaryLocal.put(SMX.getSecondaryString(), SMX.getDistance());
                    sentenceRelationMap.put(SMX.getPrimaryString(), getPrimaryLocal);
                }
            }
        });
        preRelationCounters = -100.0;
        preRelationUserCounters = -100.0;
        Double scoreRelationNewMsgToRecentMsg = 0.0;
        Double scoreRelationOldUserMsg = 0.0;
        ConcurrentMap<String, Double> getPrimaryLocal = new MapMaker().concurrencyLevel(2).makeMap();
        for (String conversationStr : conversationMatchMap.keySet()) {
            getPrimaryLocal = sentenceRelationMap.get(strF);
            Double getSecondary = getPrimaryLocal.get(conversationStr);
            if (getSecondary == null) {
                getSecondary = getScoreRelationStrF(strF, conversationStr);
                getPrimaryLocal.put(conversationStr, getSecondary);
                sentenceRelationMap.put(strF, getPrimaryLocal);
            }
            scoreRelationNewMsgToRecentMsg += getSecondary;
            System.out.println("scoreRelationNewMsgToRecentMsg: " + scoreRelationNewMsgToRecentMsg + "\n");
        }
        for (String conversationUserStr : conversationUserMatchMap.keySet()) {
            if (!strF.equals(conversationUserStr)) {
                getPrimaryLocal = sentenceRelationMap.get(strF);
                Double getSecondary = getPrimaryLocal.get(conversationUserStr);
                if (getSecondary == null) {
                    getSecondary = getScoreRelationStrF(strF, conversationUserStr);
                    getPrimaryLocal.put(conversationUserStr, getSecondary);
                    sentenceRelationMap.put(strF, getPrimaryLocal);
                }
                scoreRelationOldUserMsg += getSecondary;
            }
        }
        boolean userReponseRelated = scoreRelationOldUserMsg >= 250;
        boolean relatedReponse = scoreRelationNewMsgToRecentMsg >= 250;
        if (!userReponseRelated) {
            conversationUserMatchMap = new MapMaker().concurrencyLevel(2).makeMap();
        }
        if (!relatedReponse) {
            conversationMatchMap = new MapMaker().concurrencyLevel(2).makeMap();
        }
        ConcurrentMap<Integer, Entry<Double, SimilarityMatrix>> concurrentRelationsMap = new MapMaker().concurrencyLevel(2).makeMap();
        ConcurrentMap<Integer, Double> preRelationUserCountersMap = new MapMaker().concurrencyLevel(2).makeMap();
        for (SimilarityMatrix SMX : futureAndCacheCombineMap.values()) {
            Double scoreRelation = 500.0;
            Double scoreRelationLastUserMsg = SMX.getDistance();
            if (relatedReponse) {
                for (String conversationStr : conversationMatchMap.keySet()) {
                    Double relationNewMsg = 0.0;
                    ConcurrentMap<String, Double> getMap = sentenceRelationMap.get(conversationStr);
                    if (getMap != null) {
                        Double getdoubleValue = getMap.get(conversationStr);
                        if (getdoubleValue == null) {
                            getdoubleValue = getScoreRelationNewMsgToRecentMsg(SMX.getSecondaryString(), conversationStr);
                            getMap.put(conversationStr, getdoubleValue);
                            sentenceRelationMap.put(SMX.getSecondaryString(), getMap);
                        }
                        relationNewMsg += getdoubleValue;
                    } else {
                        relationNewMsg = getScoreRelationNewMsgToRecentMsg(SMX.getSecondaryString(), conversationStr);
                        ConcurrentMap<String, Double> localInnerMap = new MapMaker().concurrencyLevel(2).makeMap();
                        localInnerMap.put(conversationStr, relationNewMsg);
                        sentenceRelationMap.put(SMX.getSecondaryString(), localInnerMap);
                    }
                    if (userReponseRelated) {
                        relationNewMsg += scoreRelationOldUserMsg / conversationUserMatchMap.size();
                    }
                    scoreRelation += relationNewMsg;
                }
            }
            Double totalRelation = scoreRelation + scoreRelationLastUserMsg;
            if (totalRelation > preRelationCounters + preRelationUserCounters && scoreRelationLastUserMsg + (preRelationUserCounters / 10)
                    > preRelationUserCounters) {
                Entry<Double, SimilarityMatrix> localEntry = new AbstractMap.SimpleEntry(totalRelation, SMX);
                concurrentRelationsMap.put(concurrentRelationsMap.size(), localEntry);
                preRelationUserCountersMap.put(preRelationUserCountersMap.size(), preRelationUserCounters);
                preRelationCounters = scoreRelation;
                preRelationUserCounters = scoreRelationLastUserMsg;
            }
        }
        StringBuilder SB = new StringBuilder();
        int iterator = concurrentRelationsMap.size() - 1;
        System.out.println("iterator: " + iterator + "\n");
        while (iterator > -1) {
            Double preRelationUserCounterDouble = preRelationUserCountersMap.get(iterator);
            Entry<Double, SimilarityMatrix> getRelation = concurrentRelationsMap.get(iterator);
            Double result = preRelationUserCounterDouble + preRelationUserCounters;
            SB.append(getRelation.getValue().getSecondaryString()).append(" ");
            iterator--;
            System.out.println("result: " + result + "\ngetRelation.getKey(): " + getRelation.getKey() + "\npreRelationUserCounters: "
                    + preRelationUserCounters + "\npreRelationUserCounterDouble: " + preRelationUserCounterDouble + "\n");
            if (getRelation.getKey() < result) {
                break;
            }
        }
        System.out.println("Reached end: SB: " + SB.toString() + "\n: ");
        return SB.toString();
    }

    public void getSingularAnnotation(String str) {
        strAnno = new Annotation(str);
        pipeline.annotate(strAnno);
        strAnnoSentiment = new Annotation(str);
        pipelineSentiment.annotate(strAnnoSentiment);
        List<String> notactualList = new ArrayList();
        notactualList.add(str);
        ConcurrentMap<String, Annotation> jmweAnnotation = PipelineJMWESingleton.INSTANCE.getJMWEAnnotation(notactualList);
        strAnnoJMWE = jmweAnnotation.values().iterator().next();
        CoreDocument coreDocument = new CoreDocument(str);
        pipeline.annotate(coreDocument);
        coreDoc = coreDocument;
    }

    private static ConcurrentMap<String, Annotation> getMultipleJMWEAnnotation(Collection<String> str) {
        ConcurrentMap<String, Annotation> jmweAnnotation = PipelineJMWESingleton.INSTANCE.getJMWEAnnotation(str);
        return jmweAnnotation;
    }

    private static ConcurrentMap<String, Annotation> getMultiplePipelineAnnotation(Collection<String> str) {
        ConcurrentMap<String, Annotation> pipelineAnnotationMap = new MapMaker().concurrencyLevel(2).makeMap();
        for (String str1 : str) {
            Annotation strAnno1 = new Annotation(str1);
            pipelineAnnotationMap.put(str1, strAnno1);
        }
        pipeline.annotate(pipelineAnnotationMap.values());
        return pipelineAnnotationMap;
    }

    private static ConcurrentMap<String, Annotation> getMultiplePipelineSentimentAnnotation(Collection<String> str) {
        ConcurrentMap<String, Annotation> pipelineAnnotationMap = new MapMaker().concurrencyLevel(2).makeMap();
        for (String str1 : str) {
            Annotation strAnno1 = new Annotation(str1);
            pipelineAnnotationMap.put(str1, strAnno1);
        }
        pipelineSentiment.annotate(pipelineAnnotationMap.values());
        return pipelineAnnotationMap;
    }

    private Double getScoreRelationNewMsgToRecentMsg(String str, String mostRecentMsg) {
        SimilarityMatrix SMX = new SimilarityMatrix(str, mostRecentMsg);
        Callable<SimilarityMatrix> worker = new SentimentAnalyzerTest(str, mostRecentMsg, SMX,
                jmweAnnotationCache.get(str), jmweAnnotationCache.get(mostRecentMsg), pipelineAnnotationCache.get(str),
                pipelineAnnotationCache.get(mostRecentMsg), pipelineSentimentAnnotationCache.get(str),
                pipelineSentimentAnnotationCache.get(mostRecentMsg), coreDocumentAnnotationCache.get(str),
                coreDocumentAnnotationCache.get(mostRecentMsg));
        SimilarityMatrix callSMX = null;
        try {
            callSMX = worker.call();
        } catch (Exception ex) {
            Logger.getLogger(Datahandler.class
                    .getName()).log(Level.SEVERE, null, ex);
        }
        if (callSMX != null) {
            double smxDistance = callSMX.getDistance();
            return smxDistance;
        }
        return 0.0;
    }

    private Double getScoreRelationStrF(String str, String mostRecentMsg) {
        SimilarityMatrix SMX = new SimilarityMatrix(str, mostRecentMsg);
        Callable<SimilarityMatrix> worker = new SentimentAnalyzerTest(str, mostRecentMsg, SMX,
                strAnnoJMWE, jmweAnnotationCache.get(mostRecentMsg), strAnno,
                pipelineAnnotationCache.get(mostRecentMsg), strAnnoSentiment,
                pipelineSentimentAnnotationCache.get(mostRecentMsg), coreDoc, coreDocumentAnnotationCache.get(mostRecentMsg));
        SimilarityMatrix callSMX = null;
        try {
            callSMX = worker.call();
        } catch (Exception ex) {
            Logger.getLogger(Datahandler.class
                    .getName()).log(Level.SEVERE, null, ex);
        }
        if (callSMX != null) {
            double smxDistance = callSMX.getDistance();
            return smxDistance;
        }
        return 0.0;
    }

    public String mostSimilar(String toBeCompared, ConcurrentMap<Integer, String> concurrentStrings, String MostRecent) {
        similar = "";
        minDistance = 6.0;
        preRelationCounters = 0.0;
        preRelationUserCounters = 0.0;
        getSingularAnnotation(toBeCompared);
        ConcurrentMap<Integer, String> similardistances = new MapMaker().concurrencyLevel(2).makeMap();
        concurrentStrings.values().parallelStream().forEach((str) -> {
            LevenshteinDistance leven = new LevenshteinDistance(toBeCompared, str);
            double distance = leven.computeLevenshteinDistance();
            if (distance <= minDistance) {
                System.out.println("distance: " + distance + "\n");
                similardistances.put(similardistances.size() + 1, str);
            }
        });
        if (similardistances.isEmpty()) {
            return null;
        }
        Double maxDistance = 0.0;
        for (String str : similardistances.values()) {
            LinkedHashMap<String, Double> orDefault = lHMSMX.getOrDefault(str, null);
            if (orDefault != null) {
                for (Entry<String, Double> defaultEntry : orDefault.entrySet()) {
                    Double value = defaultEntry.getValue();
                    String key = defaultEntry.getKey();
                    if (value > maxDistance) {
                        Double RelationScoreLastMsg = 500.0;
                        Double RelationScoreLastUserMsg = 500.0;
                        boolean foundmatch = false;
                        if (!MostRecent.isEmpty()) {
                            LinkedHashMap<String, Double> orDefault1 = lHMSMX.getOrDefault(MostRecent, null);
                            if (orDefault1 != null) {
                                Collection<String> orDefaultstrs = orDefault1.keySet();
                                for (String strs : orDefaultstrs) {
                                    if (strs.equals(key)) {
                                        RelationScoreLastMsg = orDefault1.get(key);
                                        foundmatch = true;
                                        break;
                                    }
                                }
                            }
                            if (!foundmatch) {
                                orDefault1 = lHMSMX.getOrDefault(key, null);
                                if (orDefault1 != null) {
                                    Collection<String> orDefaultstrs = orDefault1.keySet();
                                    for (String strs : orDefaultstrs) {
                                        if (strs.equals(MostRecent)) {
                                            RelationScoreLastMsg = orDefault1.get(MostRecent);
                                            foundmatch = true;
                                            break;
                                        }
                                    }
                                }
                            }
                        }
                        if (!foundmatch) {
                            RelationScoreLastMsg = getScoreRelationNewMsgToRecentMsg(key, MostRecent);
                        }
                        RelationScoreLastUserMsg = getScoreRelationNewMsgToRecentMsg(key, toBeCompared);
                        Double totalRelation = RelationScoreLastMsg + RelationScoreLastUserMsg;
                        if (totalRelation > preRelationCounters + preRelationUserCounters) {
                            if (RelationScoreLastMsg + 500 > preRelationUserCounters && RelationScoreLastUserMsg > preRelationCounters
                                    || RelationScoreLastUserMsg + 500 > preRelationCounters && RelationScoreLastMsg > preRelationUserCounters) {
                                if (RelationScoreLastMsg > preRelationCounters && RelationScoreLastUserMsg > preRelationUserCounters) {
                                    maxDistance = value;
                                    similar = defaultEntry.getKey();
                                    preRelationCounters = RelationScoreLastMsg;
                                    preRelationUserCounters = RelationScoreLastUserMsg;
                                }
                            }
                        }
                    }
                }
            }
        }
        return similar.isEmpty() ? null : similar;
    }

    public static ConcurrentMap<Integer, String> cutContent(ConcurrentMap<Integer, String> str, boolean hlStatsMsg) {
        ConcurrentMap<Integer, String> returnlist = new MapMaker().concurrencyLevel(2).makeMap();
        str.values().parallelStream().forEach(str1 -> {
            int iend = str1.indexOf("content: ");
            if (iend != -1) {
                String trs = str1.substring(iend + 9);
                returnlist.put(returnlist.size() + 1, trs.substring(0, trs.length() - 1));
            } else if (hlStatsMsg) {
                returnlist.put(returnlist.size() + 1, str1);
            }
        });
        return returnlist;
    }

    public static ConcurrentMap<Integer, String> filterContent(ConcurrentMap<Integer, String> str) {
        ConcurrentMap<Integer, String> strlistreturn = new MapMaker().concurrencyLevel(2).makeMap();
        str.values().parallelStream().forEach(str1 -> {
            if (!str1.isEmpty() && str1.length() > 3) {
                str1 = str1.trim();
                if (str1.contains("PM*")) {
                    str1 = str1.substring(str1.indexOf("PM*") + 3);
                }
                if (str1.contains("AM*")) {
                    str1 = str1.substring(str1.indexOf("AM*") + 3);
                }
                for (Character c : str1.toCharArray()) {
                    if (c == '?' || c == '°') {
                        str1 = str1.replace("?", " <:wlenny:514861023002624001> ");
                        str1 = str1.replace("°", " <:wlenny:514861023002624001> ");
                    }
                }
                if (str1.contains("(Counter-Terrorist)")) {
                    str1 = str1.replace("(Counter-Terrorist)", " ");
                }
                if (str1.contains("(Terrorist)")) {
                    str1 = str1.replace("(Terrorist)", " ");
                }
                if (str1.contains("(Spectator)")) {
                    str1 = str1.replace("(Spectator)", " ");
                }
                if (str1.contains("*DEAD*")) {
                    str1 = str1.replace("*DEAD*", " ");
                }
                if (str1.contains("{red}")) {
                    str1 = str1.replace("{red}", " ");
                }
                if (str1.contains("{orange}")) {
                    str1 = str1.replace("{orange}", " ");
                }
                if (str1.contains("{yellow}")) {
                    str1 = str1.replace("{yellow}", " ");
                }
                if (str1.contains("{green}")) {
                    str1 = str1.replace("{green}", " ");
                }
                if (str1.contains("{lightblue}")) {
                    str1 = str1.replace("{lightblue}", " ");
                }
                if (str1.contains("{blue}")) {
                    str1 = str1.replace("{blue}", " ");
                }
                if (str1.contains("{purple}")) {
                    str1 = str1.replace("{purple}", " ");
                }
                if (str1.contains("{white}")) {
                    str1 = str1.replace("{white}", " ");
                }
                if (str1.contains("{fullblue}")) {
                    str1 = str1.replace("{fullblue}", " ");
                }
                if (str1.contains("{cyan}")) {
                    str1 = str1.replace("{cyan}", " ");
                }
                if (str1.contains("{lime}")) {
                    str1 = str1.replace("{lime}", " ");
                }
                if (str1.contains("{deeppink}")) {
                    str1 = str1.replace("{deeppink}", " ");
                }
                if (str1.contains("{slategray}")) {
                    str1 = str1.replace("{slategray}", " ");
                }
                if (str1.contains("{dodgerblue}")) {
                    str1 = str1.replace("{dodgerblue}", " ");
                }
                if (str1.contains("{black}")) {
                    str1 = str1.replace("{black}", " ");
                }
                if (str1.contains("{orangered}")) {
                    str1 = str1.replace("{orangered}", " ");
                }
                if (str1.contains("{darkorchid}")) {
                    str1 = str1.replace("{darkorchid}", " ");
                }
                if (str1.contains("{pink}")) {
                    str1 = str1.replace("{pink}", " ");
                }
                if (str1.contains("{lightyellow}")) {
                    str1 = str1.replace("{lightyellow}", " ");
                }
                if (str1.contains("{chocolate}")) {
                    str1 = str1.replace("{chocolate}", " ");
                }
                if (str1.contains("{beige}")) {
                    str1 = str1.replace("{beige}", " ");
                }
                if (str1.contains("{azure}")) {
                    str1 = str1.replace("{azure}", " ");
                }
                if (str1.contains("{yellowgreen}")) {
                    str1 = str1.replace("{yellowgreen}", " ");
                }
                str1 = str1.trim();
                if (str1.length() > 2 && (!str1.startsWith("!"))) {
                    strlistreturn.put(strlistreturn.size() + 1, str1);
                }
            }
        });
        return strlistreturn;
    }

    private ConcurrentMap<Integer, String> removeSlacks(ConcurrentMap<Integer, String> str) {
        ConcurrentMap<Integer, String> strreturn = new MapMaker().concurrencyLevel(2).makeMap();
        if (stringCache.isEmpty()) {
            return str;
        }
        Collection<String> values = stringCache.values();
        str.values().parallelStream().forEach(str1 -> {
            boolean tooclosematch = false;
            for (String strVals : values) {
                LevenshteinDistance leven = new LevenshteinDistance(strVals, str1);
                double Distance = leven.computeLevenshteinDistance();
                Double maxpermittedDistance = 2.5;
                if (Distance < maxpermittedDistance) {
                    tooclosematch = true;
                    break;
                }
            }
            if (!tooclosematch) {
                strreturn.put(strreturn.size() + 1, str1);
                System.out.println("adding strreturn str1: " + str1 + "\n");
            }
        });
        return strreturn;
    }

    private ConcurrentMap<Integer, String> annotationCacheUpdate(ConcurrentMap<Integer, String> strmap) {
        ConcurrentMap<String, Annotation> jmweAnnotation = PipelineJMWESingleton.INSTANCE.getJMWEAnnotation(strmap.values());
        for (Entry<String, Annotation> jmweitr : jmweAnnotation.entrySet()) {
            jmweAnnotationCache.put(jmweitr.getKey(), jmweitr.getValue());
        }
        ConcurrentMap<String, Annotation> Annotationspipeline = new MapMaker().concurrencyLevel(4).makeMap();
        ConcurrentMap<String, Annotation> AnnotationspipelineSentiment = new MapMaker().concurrencyLevel(4).makeMap();
        ConcurrentMap<String, CoreDocument> coreDocumentpipelineMap = getMultipleCoreDocumentsWaySuggestion(strmap.values(), pipeline);
        strmap.values().parallelStream().forEach(str -> {
            Annotation strAnno1 = new Annotation(str);
            Annotationspipeline.put(str, strAnno1);
            Annotation strAnno2 = new Annotation(str);
            AnnotationspipelineSentiment.put(str, strAnno2);
            stringCache.put(stringCache.size() + 1, str);
        });
        System.out.println("pre iterator annotation update \n");
        pipeline.annotate(Annotationspipeline.values());
        pipelineSentiment.annotate(AnnotationspipelineSentiment.values());
        Annotationspipeline.entrySet().forEach(pipelineEntry -> {
            if (pipelineEntry != null) {
                pipelineAnnotationCache.put(pipelineEntry.getKey(), pipelineEntry.getValue());
            } else {
                System.out.println("failed pipeline cache \n");
            }
        });
        AnnotationspipelineSentiment.entrySet().forEach(pipelineEntry -> {
            if (pipelineEntry != null) {
                pipelineSentimentAnnotationCache.put(pipelineEntry.getKey(), pipelineEntry.getValue());
            } else {
                System.out.println("failed sentiment cache \n");
            }
        });
        coreDocumentpipelineMap.entrySet().forEach(coreDocumentEntry -> {
            coreDocumentAnnotationCache.put(coreDocumentEntry.getKey(), coreDocumentEntry.getValue());
        });
        return strmap;
    }

    private static class AnnotationCollector<T> implements Consumer<T> {

        private static int i = 0;
        private final List<T> annotationsT = new ArrayList();

        @Override
        public final void accept(T ann) {
            annotationsT.add(ann);
        }
    }

    public final static ConcurrentMap<String, CoreDocument> getMultipleCoreDocumentsWaySuggestion(Collection<String> str, StanfordCoreNLP localNLP) {
        AnnotationCollector<Annotation> annCollector = new AnnotationCollector();
        for (final String exampleString : str) {
            System.out.println("exampleString: " + exampleString + "\n");
            localNLP.annotate(new Annotation(exampleString), annCollector);
            annCollector.i++;
            System.out.println("iterator: " + annCollector.i + "\nstr size: " + str.size() + "\n");
        }
        try {
            Thread.sleep(10000);
        } catch (InterruptedException ex) {
            Logger.getLogger(Datahandler.class.getName()).log(Level.SEVERE, null, ex);
        }
        ConcurrentMap<String, CoreDocument> annotationreturnMap = new MapMaker().concurrencyLevel(2).makeMap();
        List<CoreDocument> coreDocs = annCollector.annotationsT.stream().map(ann -> {
            try {
                return new CoreDocument(ann);
            } catch (Exception ex) {
                System.out.println(ex.getLocalizedMessage());
                return null;
            }
        }).collect(Collectors.toList());
        coreDocs.stream().forEach(CD -> {
            if (CD != null) {
                annotationreturnMap.put(CD.text(), CD);
            }
        });
        return annotationreturnMap;
    }
}
